@@ -1,7 +1,8 @@
 package Git::FastExport::Stitch;
-$Git::FastExport::Stitch::VERSION = '0.105';
+$Git::FastExport::Stitch::VERSION = '0.106';
 use strict;
 use warnings;
+use Cwd qw( cwd );
 use Carp;
 use Scalar::Util qw( blessed );
 use List::Util qw( first );
@@ -44,26 +45,37 @@ sub new {
 sub stitch {
     my ( $self, $repo, $dir ) = @_;
 
-    my $export
-        = blessed($repo) && $repo->isa('Git::FastExport')
-        ? $repo
-        : eval { Git::FastExport->new($repo) };
+    # $repo is either a Git::Repository object or a valid path
+    my $export = blessed($repo) && $repo->isa('Git::Repository')
+      ? $repo     # a Git::Repository object
+      : eval {    # assume a path
+        my $r;
+        my $orig = cwd;
+
+        # chdir and create a Git::Repository object there
+        if ( defined $repo ) {
+            chdir $repo or croak "Can't chdir to $repo: $!";
+            $r = Git::Repository->new();
+            chdir $orig or croak "Can't chdir back to $orig: $!";
+        }
+        else { die "Undefined repository path" }
+        $r;
+      };
     $@ =~ s/ at .*\z//s, croak $@ if !$export;
 
-    # initiate the Git::FastExport stream
-    $export->fast_export(qw( --progress=1 --all --date-order ))
-        if !$export->{export_fh};
-
     # do not stich a repo with itself
-    $repo = $export->{source};
+    $repo = $export->git_dir;
     croak "Already stitching repository $repo" if exists $self->{repo}{$repo};
 
     # pick the refs suffix:
-    # use basename without the .git extension or non ASCII characters
-    my $name = basename( $repo, '.git' );
+    # use base directory without the .git extension or non ASCII characters
+    my @parts = File::Spec->splitdir( ( File::Spec->splitpath( $repo, 1 ) )[1] );
+    my $name = pop @parts;
+    $name = pop @parts if $name eq '.git';
+    $name =~ s/\.git$//;
     $name =~ y/-A-Za-z0-9_/-/cs;
     $name =~ s/^-|-$//g;
-    $dir ||= $name;    # pick up a default name for the directory
+    $dir = $name if not defined $dir;
 
     # check if the name is not used already and pick a replacement if it is
     if ( exists $self->{name}{$name} ) {
@@ -72,13 +84,19 @@ sub stitch {
         $name .= "-$suffix";
     }
 
+    # initiate the Git::FastExport stream
+    my $stream =
+      $export->command(qw( fast-export --progress=1 --all --date-order ))
+      ->stdout;
+
     # set up the internal structures
     $self->{repo}{$repo}{repo}   = $repo;
     $self->{repo}{$repo}{dir}    = $dir;
-    $self->{repo}{$repo}{parser} = $export;
+    $self->{repo}{$repo}{git}    = $export;
+    $self->{repo}{$repo}{parser} = Git::FastExport->new($stream);
     $self->{repo}{$repo}{name}   = $name;
-    $self->{repo}{$repo}{block}  = $export->next_block();
-    $self->_translate_block( $repo );
+    $self->{repo}{$repo}{block}  = $self->{repo}{$repo}{parser}->next_block();
+    $self->_translate_block($repo);
 
     return $self;
 }
@@ -276,18 +294,12 @@ sub _last_alien_child {
 
 'progress 1 objects';
 
-
-
-=pod
+__END__
 
 =head1 NAME
 
 Git::FastExport::Stitch - Stitch together multiple git fast-export streams
 
-=head1 VERSION
-
-version 0.105
-
 =head1 SYNOPSIS
 
     # create a new stitch object
@@ -296,10 +308,8 @@ version 0.105
     # stitch in several git fast-export streams
     # a git directory
     $export->stitch( A => 'A' );
-    # a Git repository object
-    $export->stitch( Git->repository( Directory => 'B' ) => 'B' );
-    # a Git::FastExport object
-    $export->stitch( Git::FastExport->new('C') => 'C' );
+    # a Git::Repository object
+    $export->stitch( Git::Repository->new( work_tree => 'B' ) => 'B' );
 
     # output the stitched stream
     while ( my $block = $export->next_block() ) {
@@ -343,9 +353,8 @@ pairs) to the C<stitch()> method.
 
 Add the given C<$repo> to the list of repositories to stitch in.
 
-C<$repo> can be either a directory, or a L<Git> object (both will
-be used to instantiate a L<Git::FastExport> object) or directly a
-L<Git::FastExport> object.
+C<$repo> can be either a directory, or a L<Git::Repository> object
+(both will be used to instantiate a L<Git::FastExport> object).
 
 The optional C<$dir> parameter will be used as the relative directory
 under which the trees of the source repository will be stored in the
@@ -544,7 +553,7 @@ To ease debugging, the translated mark count starts at C<1_000_000>.
     my $commit = $self->_last_alien_child( $node, $ref, $parents )
 
 Given a node, its ref name (actually, the reference given on the
-C<commit> line of the fast-export) and a structure describing it's
+C<commit> line of the fast-export) and a structure describing its
 lineage over the various source repositories, find a suitable commit to
 which attach it.
 
@@ -554,23 +563,9 @@ This method is the heart of the stitching algorithm.
 
 B<git-stitch-repo>
 
-=head1 BUGS
-
-Please report any bugs or feature requests on the bugtracker website
-http://rt.cpan.org/NoAuth/Bugs.html?Dist=Git-FastExport or by email to
-bug-git-fastexport@rt.cpan.org.
-
-When submitting a bug or request, please include a test-file or a
-patch to an existing test-file that illustrates the bug or desired
-feature.
-
-=head1 AUTHOR
-
-Philippe Bruhat (BooK) <book@cpan.org>
-
 =head1 COPYRIGHT
 
-Copyright 2008-2014 Philippe Bruhat (BooK), All Rights Reserved.
+Copyright 2008-2015 Philippe Bruhat (BooK), All Rights Reserved.
 
 =head1 LICENSE
 
@@ -578,9 +573,3 @@ This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
 =cut
-
-
-__END__
-
-# ABSTRACT: Stitch together multiple git fast-export streams
-
